@@ -29,8 +29,6 @@
 #define md_operand(x)
 
 /* Call md_pcrel_from_section, not md_pcrel_from.  */
-struct fix;
-extern long md_pcrel_from_section (struct fix *, segT);
 #define MD_PCREL_FROM_SECTION(FIX, SEC) md_pcrel_from_section (FIX, SEC)
 
 /* Permit temporary numeric labels.  */
